<?php
// phpXMLDOM - an XML DOM Implementation
//
// This Software and all associated files are released unter the 
// GNU Public License (GPL), see LICENSE for details.
//
// $Id: Attr.php,v 1.3 2002/02/13 21:21:47 thomi Exp $

/**
 * Class representing the DOM Attr interface.
 *
 * @package  phpXD
 * @author   Thomas Dohmke <thomas@dohmke.de>
 * @version  $Revision: 1.3 $
 */
class Attr extends Node {
  /**
   * The name of the attribute.<br>
   * DOM-Level 1
   *
   * @private
   * @type    string
   */
  var $name;
  
  /**
   * A link to the Element object that owns this attribute.<br>
   * DOM-Level 2
   *
   * @private
   * @type    Node
   */
  var $ownerElement;
  
  /**
   * If this attribute was explicity set in the XML source for the parent 
   * element or it is a default value in the DTD (currently not supported by
   * phpXD) specified is true.<br>
   * DOM-Level 1
   *
   * @private
   * @type    boolean
   */
  var $specified;
  
  /**
   * The value of the attribute.<br>
   * DOM-Level 1
   *
   * @private
   * @type    string
   */
  var $value;
  
  /**
   * Constructor of the class.
   *
   * @public
   */
  function Attr() {
    Node::Node();
    $this->nodeType = ATTRIBUTE_NODE;
    $this->specified = true;
    $this->name =& $this->nodeName;
  }
  
  /**
   * Returns the name of the attribute.<br>
   * DOM-Level 1
   *
   * @public
   * @returns string
   */
  function getName() {
    return $this->name;
  }
  
  /**
   * Returns a reference to the owner of the attribute.<br>
   * DOM-Level 2
   *
   * @public
   * @returns Node
   */
  function &getOwnerElement() {
    return $this->ownerElement;
  }
  
  /**
   * Returns true if the attribute was set explicity in the XML Source or in
   * the DTD (not supported).<br>
   * DOM-Level 1
   *
   * @public
   * @returns boolean
   */
  function getSpecified() {
    return $this->specified;
  }
  
  /**
   * Return the value of the attribute.<br>
   * DOM-Level 1
   *
   * @public
   * @returns string
   */
  function getValue() {
    if (isset($this->value)) {
      return $this->value;
    }
  }
  
  /**
   * Sets the value of the attribute<br>
   * DOM-Level 1
   *
   * @public
   * @param  $value <code>string</code>
   * @returns void
   */
  function setValue($value) {
    if (isset($this->childNodes)) {
      $child =& $this->firstChild;
      while (isset($child)) {
        removeChild($child);
        $child =& $this->firstChild;
      }
    }
    
    $new = new Text();
    $new->nodeValue = $value;
    $new->ownerDocument =& $this->ownerDocument;
    
    $this->firstChild =& $new;
    $this->lastChild =& $this->firstChild;
    $this->nodeValue =& $this->firstChild->nodeValue;
    $this->value =& $this->nodeValue;
    $this->specified = true;
  }
  
  /**
   * Sets the value of the attribute (inherited function from Node interface)
   * <br>DOM-Level 1
   *
   * @public
   * @param   $nodeValue <code>string</code>
   * @returns void
   */
  function setNodeValue($nodeValue) {
    $this->setValue($nodeValue);
  }

  /**
   * This inherited function from Node interface is not supported by the Attr
   * interface.<br>
   * DOM-Level 1
   *
   * @public
   * @returns NOT_SUPPORTED_ERR
   */
  function &appendChild(&$newChild) {
    return NOT_SUPPORTED_ERR;
  }
  
  /**
   * This inherited function from Node interface is not supported by the Attr
   * interface.<br>
   * DOM-Level 1
   *
   * @public
   * @returns NOT_SUPPORTED_ERR
   */
  function &insertBefore(&$newChild, &$refChild) {
    return NOT_SUPPORTED_ERR;
  }
  
  /**
   * This inherited function from Node interface is not supported by the Attr
   * interface.<br>
   * DOM-Level 1
   *
   * @public
   * @returns NOT_SUPPORTED_ERR
   */
  function &replaceChild($newChild, $oldChild) {
    return NOT_SUPPORTED_ERR;
  }
}
?>
