<?php
// phpXMLDOM - an XML DOM Implementation
//
// This Software and all associated files are released unter the 
// GNU Public License (GPL), see LICENSE for details.
//
// $Id: CharacterData.php,v 1.3 2002/02/13 21:21:47 thomi Exp $

/**
 * Class representing the DOM CharacterData interface.
 *
 * @package  phpXD
 * @author   Thomas Dohmke <thomas@dohmke.de>
 * @version  $Revision: 1.3 $
 */
class CharacterData extends Node {
  /**
   * The "raw" data of the CharacterData node.<br>
   * DOM-Level 1
   *
   * @private
   * @type    string
   */
  var $data;
  
  /**
   * The size of the string stored in data.<br>
   * DOM-Level 1
   *
   * @private
   * @type    int
   */
  var $length;
  
  /**
   * Constructor of the class.
   *
   * @public
   */
  function CharacterData() {
    Node::Node();
    $this->data =& $this->nodeValue;
  }
  
  /**
   * Returns the data stored in this node.<br>
   * DOM-Level 1
   *
   * @public
   * @returns string
   */
  function getData() {
    return $this->data;
  }
  
  /**
   * Sets the data stored in this node.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $data <code>string</code>
   * @returns string
   */
  function setData($data) {
    $this->nodeValue = $data;
  }
  
  /**
   * Returns the length of the data stored in this node.<br>
   * DOM-Level 1
   *
   * @public
   * @returns string
   */
  function getLength() {
    return strlen($this->data);
  }
  
  /**
   * Appends contents of the arg parameter to the current contents of the data
   * attribute.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $arg <code>string</code>
   * @returns string
   */
  function appendData($arg) {
    $this->nodeValue .= $arg;
  }
  
  /**
   * Truncates the data attribute; It removes count characters, starting at
   * the offset position.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $offset <code>int</code>
   * @param   $count  <code>int</code>
   * @returns string
   */
  function deleteData($offset, $count) {
    $this->nodeValue = substr($this->nodeValue, 0, $offset-1). 
       substr($this->nodeValue, $offset+count);
  }
  
  /**
   * Takes a string, splits the data attributes current contents at the given
   * offset, then inserts the string arg between the two substrings.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $offset <code>int</code>
   * @param   $arg    <code>string</code>
   * @returns string
   */
  function insertData($offset, $arg) {
      $this->nodeValue = substr($this->nodeValue, 0, $offset-1).$arg.
         substr($this->nodeValue, $offset);
  }
  
  /**
   * Replaces a substring within the data attribute with another string.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $offset <code>int</code>
   * @param   $count  <code>int</code>
   * @param   $arg    <code>string</code>
   * @returns string
   */
  function replaceData($offset, $count, $arg) {
    if ($offset + $count > strlen($this->nodeValue)) {
      $count = strlen($this->nodeValue) - $offset;
    }
      
    $this->nodeValue = substr($this->nodeValue, 0, $offset-1).
       substr($arg, 0, $count) . substr($this->nodeValue, $offset+$count);
  }
  
  /**
   * Returns a string that contains a subset of the string stored in the data
   * attribute.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $offset <code>int</code>
   * @param   $count  <code>int</code>
   * @returns string
   */
  function substringData($offset, $count) {
    return substr($this->nodeValue, $offset, $count);
  }
}
?>
