<?php
// phpXMLDOM - an XML DOM Implementation
//
// This Software and all associated files are released unter the 
// GNU Public License (GPL), see LICENSE for details.
//
// $Id: DOMImplementation.php,v 1.5 2002/02/13 21:21:47 thomi Exp $

/**
 * Class representing the DOM DOMImplementation interface.
 *
 * @package  phpXD
 * @author   Thomas Dohmke <thomas@dohmke.de>
 * @version  $Revision: 1.5 $
 */
class DOMImplementation {
  /**
   * Tests to see if the DOM implementation supports a given named feature 
   * package.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $feature <code>string</code>
   * @param   $version <code>string</code>
   * @returns boolean
   */
  function hasFeature($feature, $version = "1.0") {
    if (($feature == "XML") || ($feature == "Core")) {
      if (($version == "1.0") ||
	  ($version = "2.0")) {
        return true;
      }
    }
    return false;
  }

  /**
   * Creates a new, empty Document with the given document type.<br>
   * DOM-Level 2
   *
   * @public
   * @param   $namespaceURI  <code>string</code>
   * @param   $qualifiedName <code>string</code>
   * @param   $doctype       <a href="DocumentType.html">DocumentType</a>
   * @returns Document
   */
  function &createDocument($namespaceURI, $qualifiedName, &$doctype) {
    if (empty($qualifiedName)) {
      return NAMESPACE_ERR;
    }

    $documentElement = new Element();
    $documentElement->nodeName = $qualifiedName;
    if (!(strpos($qualifiedName, ":") === false)) {
      $prefix = substr($qualifiedName, 0, strpos($qualifiedName, ":"));
      $localName = substr($qualifiedName, strpos($qualifiedName, ":")+1);

      if (empty($prefix) || empty($localName)) {
        return NAMESPACE_ERR;
      }

      if (empty($namespaceURI)) {
	return NAMESPACE_ERR;
      }
      
      if (($namespaceURI != "http://www.w3.org/XML/1998/namespace") && 
	  ($prefix == "xml")) {
	return NAMESPACE_ERR;
      }

      $documentElement->namespaceURI = $namespaceURI;
      $documentElement->localName = $localName;
      $documentElement->prefix = $prefix;
    }

    if (!empty($doctype->ownerDocument)) {
      return WRONG_DOCUMENT_ERR;
    }

    $doc = new Document();
    $documentElement->ownerDocument =& $doc;
    $doc->documentElement =& $documentElement;
    if ($doctype != null) {
      $doc->doctype =& $doctype;
      $doc->doctype->ownerDocument =& $doc;
      $doc->doctype->namespaceURI = $namespaceURI;
      $doc->namespaceURI = $namespaceURI;
    }
    return $doc;
  }

  /**
   * Creates an empty DocumentType node that is not associated with any
   * document.<br>
   * DOM-Level 2
   *
   * @public
   * @param   $qualifiedName <code>string</code>
   * @param   $publicId      <code>string</code>
   * @param   $systemId      <code>string</code>
   * @returns DocumentType
   */
  function createDocumentType($qualifiedName, $publicId, $systemId) {
    $dtd = new DocumentType();

    if (!(strpos($qualifiedName, ":") === false)) {
      $prefix = substr($qualifiedName, 0, strpos($qualifiedName, ":"));
      $localName = substr($qualifiedName, strpos($qualifiedName, ":")+1);

      if (empty($prefix) || empty($localName)) {
        return NAMESPACE_ERR;
      }

      $dtd->prefix = $prefix;
      $dtd->localName = $localName;
    }
    else {
      if (empty($qualifiedName)) {
        return NAMESPACE_ERR;
      }
    }

    $dtd->publicId = $publicId;
    $dtd->systemId = $systemId;
    $dtd->nodeName = $qualifiedName;
    return $dtd;
  }
}
?>
