<?php
// phpXMLDOM - an XML DOM Implementation
//
// This Software and all associated files are released unter the 
// GNU Public License (GPL), see LICENSE for details.
//
// $Id: DocumentType.php,v 1.3 2002/02/13 21:21:47 thomi Exp $

/**
 * Class representing the DOM DocumentType interface.
 *
 * @package  phpXD
 * @author   Thomas Dohmke <thomas@dohmke.de>
 * @version  $Revision: 1.3 $
 */
class DocumentType extends Node {
  /**
   * A list off all defined elements and attributes. This is not DOM
   * standard!
   *
   * @private
   * @type   NamedNodeMap
   */
  var $elements;

  /**
   * A list of all generall entities declared in the DTD.<br>
   * DOM-Level 1
   *
   * @private
   * @type   NamedNodeMap
   */
  var $entities;

  /**
   * The name of the DTD.<br>
   * DOM-Level 1
   *
   * @private
   * @type   string
   */
  var $name;
  
  /**
   * Contains a list of XML notation declarations.<br>
   * DOM-Level 1
   *
   * @private
   * @type   NamedNodeMap
   */
  var $notations;
  
  /**
   * Contains the documents internat subset as a string value.<br>
   * DOM-Level 2
   *
   * @private
   * @type   string
   */
  var $internalSubset;
  
  /**
   * The public identifier of the external subset.<br>
   * DOM-Level 2
   *
   * @private
   * @type   string
   */
  var $publicId;
  
  /**
   * The system identifier of the external subset.<br>
   * DOM-Level 2
   *
   * @private
   * @type   string
   */
  var $systemId;
  
  /**
   * Constructor of the class.<br>
   *
   * @public
   */
  function DocumentType() {
    Node::Node();
    $this->nodeType = DOCUMENT_TYPE_NODE;
    $this->name =& $this->nodeName;
    
    $this->notations = new NamedNodeMap();
    $this->elements = new NamedNodeMap();
    $this->entities = new NamedNodeMap();
  }

  /**
   * Returns a NamedNodeMap containing all general entities declared in the
   * DTD.<br>
   * DOM-Level 1
   *
   * @public
   * @returns NamedNodeMap
   */
  function &getEntities() {
    return $this->entities;
  }
  
  /**
   * Returns the internal subset.<br>
   * DOM-Level 2
   *
   * @public
   * @returns string
   */
  function getInternalSubset() {
    return $this->internalSubset;
  }
  
  /**
   * Returns the name of the DTD.<br>
   * DOM-Level 1
   *
   * @public
   * @returns string
   */
  function getName() {
    return $this->name;
  }
  
  /**
   * Returns a NamedNodeMap containing all notations declared in the 
   * DTD.<br>
   * DOM-Level 1
   *
   * @public
   * @returns NamedNodeMap
   */
  function &getNotations() {
    return $this->notation;
  }
  
  /**
   * Returns the public identifier of an external subset.<br>
   * DOM-Level 2
   *
   * @public
   * @returns string
   */
  function getPublicId() {
    return $this->publicId;
  }
  
  /**
   * Returns the system identifier of an external subset.<br>
   * DOM-Level 2
   *
   * @public
   * @returns string
   */
  function getSystemId() {
    return $this->systemId;
  }
}
?>
