<?php
// phpXMLDOM - an XML DOM Implementation
//
// This Software and all associated files are released unter the 
// GNU Public License (GPL), see LICENSE for details.
//
// $Id: NamedNodeMap.php,v 1.3 2002/02/13 21:21:47 thomi Exp $

/**
 * Class representing the DOM NamedNodeMap interface
 *
 * @package  phpXD
 * @author   Thomas Dohmke <thomas@dohmke.de>
 * @version  $Revision: 1.3 $
 */
class NamedNodeMap {
  /**
   * Contains all items of tbe NamedNodeMap.<br>
   *
   * @private
   * @type    array
   */
  var $nodes;
  
  /**
   * The number of nodes in this map.<br>
   * DOM-Level 1
   *
   * @private
   * @type    int
   */
  var $length = 0;
  
  /**
   * Returns the number of nodes in the map.<br>
   * DOM-Level 1
   *
   * @public
   * @returns int
   */
  function getLength() {
    return $this->length;
  }
  
  /**
   * Retrieves a node specified by name<br>
   * DOM-Level 1
   *
   * @public
   * @param   $name <code>string</code>
   * @returns Node
   */
  function &getNamedItem($name) {
    return $this->nodes[$name];
  }
  
  /**
   * Returns the indexth item in this map.<br> 
   * DOM-Level 1
   *
   * @public
   * @param   $index <code>int</code>
   * @returns Node
   */
  function &item($index) {
    $count = 0;
    reset($this->nodes);
    foreach($this->nodes as $key => $value) {
      if ($count == $index) {
        return $this->nodes[$key];
      }
      $count++;
    }
    return null;
  }
  
  /**
   * Removes a node specified by name.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $name <code>string</code>
   * @returns Node
   */
  function &removeNamedItem($name) {
    if (isset($this->nodes[$name])) {
      $removed =& $this->nodes[$name];
      unset($this->nodes[$name]);
      $this->length--;
      return $removed;
    }
    else {
      return NOT_FOUND_ERR;
    }
  }

  /**
   * Adds a node using its nodeName attribute.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $arg <a href="Node.html">Node</a>
   * @returns Node
   */
  function &setNamedItem(&$arg) {
    if (!isset($this->nodes[$arg->nodeName])) {
      $this->length++;
      $save = null;
    }
    else {
      $save =& $this->nodes[$arg->nodeName];
    }
    $this->nodes[$arg->nodeName] =& $arg;
    return $save;
  }

  /**
   * Retrieves a node specified by local name and namespace URI.<br>
   * DOM-Level 2
   *
   * @public
   * @param   $namespaceURI <code>string</code>
   * @param   $localName    <code>string</code>
   * @returns Node
   */
  function &getNamedItemNS($namespaceURI, $localName) {
    return $this->getNamedItem($namespaceURI.":".$localName);
  }

  /**
   * Removes a node specified by local name and namespace URI.<br>
   * DOM-Level 2
   *
   * @public
   * @param   $namespaceURI <code>string</code>
   * @param   $localName    <code>string</code>
   * @returns Node
   */
  function &removeNamedItemNS($namespaceURI, $localName) {
    return $this->removeNamedItem($namespaceURI.":".$localName);
  }

  /**
   * Adds a node using its $namespaceURI and $localName<br>
   * DOM-Level 2
   *
   * @public
   * @param   $arg <a href="Node.html">Node</a>
   * @returns Node
   */
  function &setNamedItemNS(&$arg) {
    if (!isset($this->nodes[$arg->namespaceURI.":".$arg->localName])) {
      $this->length++;
      $save = null;
    }
    else {
      $save =& $this->nodes[$arg->namespaceURI.":".$arg->localName];
    }
    $this->nodes[$arg->namespaceURI.":".$arg->localName] =& $arg;
    
    return $save;
  }
}
?>
