<?php
// phpXMLDOM - an XML DOM Implementation
//
// This Software and all associated files are released unter the 
// GNU Public License (GPL), see LICENSE for details.
//
// $Id: Node.php,v 1.4 2002/02/13 21:21:47 thomi Exp $

/**
* @const ELEMENT_NODE
*/
define("ELEMENT_NODE", 1);

/**
 * @const ATTRIBUTE_NODE
 */
define("ATTRIBUTE_NODE", 2);

/**
 * @const TEXT_NODE
 */
define("TEXT_NODE", 3);

/**
 * @const CDATA_SECTION_NODE
 */
define("CDATA_SECTION_NODE", 4);

/**
 * @const ENTITY_REFERENCE_NODE
 */
define("ENTITY_REFERENCE_NODE", 5);

/**
 * @const ENTITY_NODE
 */
define("ENTITY_NODE", 6);

/**
 * @const PROCESSING_INSTRUCTION_NODE
 */
define("PROCESSING_INSTRUCTION_NODE", 7);

/**
 * @const COMMENT_NODE
 */
define("COMMENT_NODE", 8);

/**
 * @const DOCUMENT_NODE
 */
define("DOCUMENT_NODE", 9);

/**
 * @const DOCUMENT_TYPE_NODE
 */
define("DOCUMENT_TYPE_NODE", 10);

/**
 * @const DOCUMENT_FRAGMENT_NODE
 */
define("DOCUMENT_FRAGMENT_NODE", 11);

/**
 * @const NOTATION_NODE
 */
define("NOTATION_NODE", 12);

/**
 * Class representing the DOM Node interface
 *
 * @package  phpXD
 * @author   Thomas Dohmke <thomas@dohmke.de>
 * @version  $Revision: 1.4 $
 */
class Node {
  /**
   * Identifier to compare two nodes.
   *
   * This ID is necessary to compare two nodes, because the operator == 
   * is not implemented for objects/references in PHP4.
   *
   * @private
   * @type    string
   */
  var $nodeID;
  
  /**
   * Provides access to a list of Attr objects in a NamedNodeMap.<br>
   * DOM-Level 1
   *
   * @private
   * @type    NamedNodeMap
   */
  var $attributes;
  
  /**
   * A NodeList containing a reference to every child in this Node.<br>
   * DOM-Level 1
   *
   * @private
   * @type    NodeList
   */
  var $childNodes;
  
  /**
   * Points to the head of the linked list of children in this node.<br>
   * DOM-Level 1
   *
   * @private
   * @type    Node
   */
  var $firstChild;
  
  /**
   * Points to the end of the linked list of children in the node.<br>
   * DOM-Level 1
   *
   * @private
   * @type    Node
   */
  var $lastChild;
  
  /**
   * Points to the next Node in the siblings list.<br>
   * DOM-Level 1
   *
   * @private
   * @type    Node
   */
  var $nextSibling;
  
  /**
   * Depending on the object type (nodeType), this attribute may map to 
   * another attribute of the object or a constant string.<br>
   * DOM-Level 1
   *
   * @private
   * @type    string
   */
  var $nodeName;
  
  /**
   * Contains a value that indicates the true type of the Node.<br>
   * DOM-Level 1
   *
   * @private
   * @type    int
   */
  var $nodeType;
  
  /**
   * Depending on the nodeType, this attribute contains null or the 
   * content/value of the node.<br>
   * DOM-Level 1
   *
   * @private
   * @type    string
   */
  var $nodeValue;
  
  /**
   * Points to the Document used to create this Node object.<br>
   * DOM-Level 1
   *
   * @private
   * @type    Document
   */
  var $ownerDocument = null;
  
  /**
   * Points to the parent of this node.<br>
   * DOM-Level 1
   *
   * @private
   * @type    Node
   */
  var $parentNode;
  
  /**
   * Points to the previous Node in the siblings list.<br>
   * DOM-Level 1
   *
   * @private
   * @type    Node
   */
  var $previousSibling;
  
  /**
   * Contains the local part of the fully qualified node name.<br>
   * DOM-Level 2
   *
   * @private
   * @type    string
   */
  var $localName;
  
  /**
   * The namespace URI given to this Node object at creation time, null
   * if no namespace was given.<br>
   * DOM-Level 2
   *
   * @private
   * @type    string
   */
  var $namespaceURI;
  
  /**
   * The namespace prefix of this Node, used for nodes that support namespace
   * prefixes.<br>
   * DOM-Level 2
   *
   * @private
   * @type    string
   */
  var $prefix;

  /**
   * Mark this node as a readonly node.
   *
   * @private
   * @type    boolean
   */
  var $readonly = false;

  /**
   * Constructor of the class.
   *
   * @public
   */
  function Node() {
      $this->nodeID = microtime()." ".rand(); 
  }

  /**
   * Provides access to a list of Attr objects in a NamedNodeMap.<br>
   * DOM-Level 1
   *
   * @public
   * @returns NamedNodeMap
   */
  function &getAttributes() {
    return $this->attributes;
  }
  
  /**
   * Returns a NodeList containing a reference to every child in this Node.<br>
   * DOM-Level 1
   *
   * @public
   * @returns NodeList
   */
  function &getChildNodes() {
    return $this->childNodes;
  }
  
  /**
   * Returns a reference to the head of the linked list of children
   * in this node.<br>
   * DOM-Level 1
   *
   * @public
   * @returns Node
   */
  function &getFirstChild() {
    return $this->firstChild;
  }
  
  /**
   * Returns a reference to the end of the linked list of children
   * in this node.<br>
   * DOM-Level 1
   *
   * @public
   * @returns Node
   */
  function &getLastChild() {
    return $this->lastChild;
  }
  
  /**
   * Returns a reference to the next node in the siblings list.<br>
   * DOM-Level 1
   *
   * @public
   * @returns Node
   */
  function &getNextSibling() {
    return $this->nextSibling;
  }
  
  /**
   * Returns the name of the node or null if the node has no name.<br>
   * DOM-Level 1
   *
   * @public
   * @returns string
   */
  function getNodeName() {
    return $this->nodeName;
  }
  
  /**
   * Returns the type of the node.<br>
   * DOM-Level 1
   *
   * @public
   * @returns int
   */
  function getNodeType() {
    return $this->nodeType;
  }
  
  /**
   * Returns the value of the node or null if the node has no value.<br>
   * DOM-Level 1
   *
   * @public
   * @returns string
   */
  function getNodeValue() {
    return $this->nodeValue;
  }
  
  /**
   * Sets the value of this node.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $nodeValue <code>string</code>
   * @returns void
   */
  function setNodeValue($nodeValue) {
    if (($this->nodeType == ELEMENT_NODE) ||
        ($this->nodeType == ENTITY_REFERENCE_NODE) ||
        ($this->nodeType == ENTITY_NODE) ||
        ($this->nodeType == DOCUMENT_NODE) ||
        ($this->nodeType == DOCUMENT_TYPE_NODE) ||
        ($this->nodeType == DOCUMENT_FRAGMENT_NODE) ||
        ($this->nodeType == NOTATION)) {
      return NO_MODIFICATION_ALLOWED_ERR;
    }
    
    $this->nodeValue = $nodeValue;
    return null;
  }
  
  /**
   * Returns a reference to the Document used to create this node.<br>
   * DOM-Level 1
   *
   * @public
   * @returns Document
   */
  function &getOwnerDocument() {
    return $this->ownerDocument;
  }
  
  /**
   * Returns the parent of this node.<br>
   * DOM-Level 1
   *
   * @public
   * @returns Node
   */
  function &getParentNode() {
    return $this->parentNode;
  }
  
  /**
   * Returns the previous node in the siblings list.<br>
   * DOM-Level 1
   *
   * @public
   * @returns Node
   */
  function &getPreviousSibling() {
    return $this->previousSibling;
  }
  
  /**
   * Appends the $newChild node to the end of the child list if the node is
   * a permitted child of this node.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $newChild <a href="Node.html">Node</a>
   * @returns Node
   */
  function &appendChild(&$newChild) {
    if (($newChild->nodeType == DOCUMENT_FRAGMENT_NODE) && 
        ($this->nodeType != DOCUMENT_FRAGMENT_NODE)) {
      for ($i = 0; $i < $newChild->childNodes->getLength(); $i++) {
        $save = &$this->appendChild($newChild->childNodes->item($i));
      }
      return $save;
    }

    $check = $this->checkHierarchy($newChild);
    if ($check > 0) {
      return $check;
    }

    $check = $this->checkDocument($newChild);
    if ($check > 0) {
      return $check;
    }
    
    if (isset($newChild->parentNode)) {
      $result = $newChild->parentNode->removeChild($newChild);
    }
    
    $newChild->parentNode = &$this;
    unset($newChild->nextSibling);
    unset($newChild->previousSibling);
    
    if (!isset($this->firstChild)) {
      $this->firstChild = &$newChild;
      $this->lastChild = &$newChild;
      $this->childNodes = new NodeList();
    }
    else {
      $this->lastChild->nextSibling = &$newChild;
      $newChild->previousSibling = &$this->lastChild;
      $this->lastChild = &$newChild;
    }
    $this->childNodes->insertNode($newChild);
    return $newChild;
  }

  /**
   * Returns a copy of the node without the parent node; If $deep is true,
   * all children are also copied.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $deep <code>boolean</code>
   * @param   $ownerDocument Set the ownerDocument of all cloned nodes.
   * @returns Node        
   */
  function &cloneNode($deep) {
    return $this->cloneNode($deep, null);
  }

  /**
   * Internal function for cloning a node.
   *
   * @private
   * @param   $deep <code>boolean</code>
   * @param   $ownerDocument Set the ownerDocument of all cloned nodes.
   * @returns Node        
   */
  function &cloneNode($deep, $ownerDocument = null) {
    $clone = $this;
    $clone->Node();
    if ($ownerDocument != null) {
      $clone->ownerDocument = $ownerDocument;
    }
    unset($clone->attributes);
    unset($clone->childNodes);
    unset($clone->firstChild);
    unset($clone->lastChild);
    unset($clone->nextSibling);
    unset($clone->parentNode);
    unset($clone->previousSibling);
    
    if (isset($this->attributes)) {
      $clone->attributes = new NamedNodeMap();
      
      unset($oldAttr);
      for ($i = 0; $i < $this->attributes->getLength(); $i++) {
        $attr =& $this->attributes->item($i);
        $newAttr[$i] =& $attr->cloneNode(true, $ownerDocument);
        $newAttr[$i]->specified = true;
        if (!isset($oldAttr)) {
          $clone->firstAttr = &$newAttr[$i];
        }
        else {
          $oldAttr->nextSibling = &$newAttr[$i];
          $newAttr[$i]->previousSibling = &$oldAttr;
        }
        $clone->attributes->setNamedItem($newAttr[$i]);
        $oldAttr = &$newAttr[$i];
      }
      $clone->lastAttr = &$oldAttr;
    }
    
    if ($deep) {
      if (isset($this->childNodes)) {
        $clone->childNodes = new NodeList();
        $oldChild = null;
        for ($i = 0; $i < $this->childNodes->getLength(); $i++) {
          $child =& $this->childNodes->item($i);
          $newChild[$i] =& $child->cloneNode($deep, $ownerDocument);
          if (!isset($oldChild)) {
            $clone->firstChild = &$newChild[$i];
          }
          else {
            $oldChild->nextSibling = &$newChild[$i];
            $newChild[$i]->previousSibling = &$oldChild;
          }
          $clone->childNodes->insertNode($newChild[$i]);
          $oldChild = &$newChild[$i];
        }
        $clone->lastChild = &$oldChild;
      }
    }
    
    return $clone;
  }

  /**
   * This method provides a quick way to check if a node has children.<br>
   * DOM-Level 1
   *
   * @public
   * @returns boolean
   */
  function hasChildNodes() {
    if (isset($this->firstChild)) {
      return true;
    }
    return false;
  }

  /**
   * Inserts the Node object $newChild into the child list of the parent node;
   * The $refChild parameter allows you to specify where to insert the new 
   * node; If $refChild is null, newChild is inserted at the end of the child
   * list; If $newChild is linked into the document tree, it is removed before 
   * it is inserted in its new position.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $newChild <a href="Node.html">Node</a>
   * @param   $refChild <a href="Node.html">Node</a>
   * @returns Node
   */
  function &insertBefore(&$newChild, &$refChild) {
    if (($newChild->nodeType == DOCUMENT_FRAGMENT_NODE) && 
        ($this->nodeType != DOCUMENT_FRAGMENT_NODE)) {
      for ($i = 0; $i < $newChild->childNodes->getLength(); $i++) {
        $save = &$this->insertBefore($newChild->childNodes->item($i), 
                                     $refChild);
      }
      return $save;
    }
    
    $check = $this->checkHierarchy($newChild);
    if ($check > 0) {
      return $check;
    }

    $check = $this->checkDocument($newChild);
    if ($check > 0) {
      return $check;
    }

    if (isset($newChild->parentNode)) {
      $result = $newChild->parentNode->removeChild($newChild);
    }
    
    $newChild->parentNode = &$this;
    $newChild->ownerDocument = &$this->ownerDocument;
    unset($newChild->nextSibling);
    unset($newChild->previousSibling);
                
    if (isset($this->childNodes)) {
      for ($i = 0; $i < $this->childNodes->getLength(); $i++) {
        $child =& $this->childNodes->item($i);
        if ($child->nodeID == $refChild->nodeID && !$inserted) {
          if ($i > 0) {
            $prevChild = &$this->childNodes->item($i-1);
            $prevChild->nextSibling = &$newChild;
            $newChild->previousSibling = &$prevChild;
          } 
          else {
            $this->firstChild = &$newChild;
          }
          $newChild->nextSibling = &$child;
          $child->previousSibling = &$newChild;
          $this->childNodes->insertNode($newChild, $i);
          return $newChild;
        }
      }
    }
    else {
      $this->firstChild = &$newChild;
      $this->lastChild = &$this->firstChild;
      $this->childNodes = new NodeList();
      $this->childNodes->insertNode($newChild);
      return $newChild;
    }
    
    if (!isset($refChild)) {
      $this->lastChild->nextSibling = &$newChild;
      $newChild->previousSibling = &$this->lastChild;
      $this->lastChild = &$newChild;
      $this->childNodes->insertNode($newChild);
      return $newChild;
    }
    else {
      return NOT_FOUND_ERR;
    }
  }
        
  /**
   * Unlinks the $oldChild node from the child list.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $oldChild <a href="Node.html">Node</a>
   * @returns Node
   */
  function &removeChild($oldChild) {
    if (isset($this->childNodes)) {
      for ($i = 0; $i < $this->childNodes->getLength(); $i++) {
        $child =& $this->childNodes->item($i);
        if ($child->nodeID == $oldChild->nodeID) {
          if ($i > 0) {
            $prevChild = &$this->childNodes->item($i-1);
            $prevChild->nextSibling = &$child->nextSibling;
            if (isset($child->nextSibling)) {
              $child->nextSibling->previousSibling = &$prevChild;
            }
            else {
              $this->lastChild = &$prevChild;
            }
          } 
          else {
            $this->firstChild = &$child->nextSibling;
          }
          $removedNode = &$this->childNodes->nodes[$i];
          $this->childNodes->removeNode($i);
          return $removedNode;
        }
      }
    }
    return NOT_FOUND_ERR;
  }

  /**
   * Replaces the child node $oldChild with $newChild; If $newChild is linked 
   * into the document tree, it is removed before the replace is performed.<br>
   * DOM-Level 1
   *
   * @public
   * @param   $newChild <a href="Node.html">Node</a>
   * @param   $oldChild <a href="Node.html">Node</a>
   * @returns Node
   */
  function &replaceChild($newChild, $oldChild) {
    if (($newChild->nodeType == DOCUMENT_FRAGMENT_NODE) && 
        ($this->nodeType != DOCUMENT_FRAGMENT_NODE)) {
      for ($i = 0; $i < $newChild->childNodes->getLength(); $i++) {
        $save = &$this->insertBefore($newChild->childNodes->item($i), 
                                     $oldChild);
      }
      return $this->removeChild($oldChild);
    }

    $check = $this->checkHierarchy($newChild);
    if ($check > 0) {
      return $check;
    }

    $check = $this->checkDocument($newChild);
    if ($check > 0) {
      return $check;
    }

    if (isset($newChild->parentNode)) {
      $result = $newChild->parentNode->removeChild($newChild);
    }
    
    $newChild->parentNode = &$this;
    
    if (isset($this->childNodes)) {
      for ($i = 0; $i < $this->childNodes->getLength(); $i++) {
        $child =& $this->childNodes->item($i);
        if ($child->nodeID == $oldChild->nodeID) {
          if ($i > 0) {
            $prevChild = &$this->childNodes->item($i-1);
            $prevChild->nextSibling = &$newChild;
          } 
          else {
            $this->firstChild = &$newChild;
          }
          $newChild->nextSibling = &$child->nextSibling;
          $newChild->previousSibling = &$child->previousSibling;
          if (isset($newChild->nextSibling)) {
            $newChild->nextSibling->previousSibling = &$newChild;
          }
          else {
            $this->lastChild = &$newChild;
          }
          $this->childNodes->replaceNode($newChild, $i);
          return $oldChild;
        }
      }
    }
    return NOT_FOUND_ERR;
  }
  
  /**
   * Returns the local part of the fully qualified node name.<br>
   * DOM-Level 2
   *
   * @public
   * @returns string
   */
  function getLocalName() {
    return $this->localName;
  }
  
  /**
   * Returns the namespace URI of this node.<br>
   * DOM-Level 2
   *
   * @public
   * @returns string
   */
  function getNamespaceURI() {
    return $this->namespaceURI;
  }
  
  /**
   * Returns the namespace prefix of this node.<br>
   * DOM-Level 2
   *
   * @public
   * @returns string
   */
  function getPrefix() {
    return $this->prefix;
  }
  
  /**
   * Changes the prefix of this node; For ELEMENT_NODE and ATTRIBUTE_NODE
   * type nodes, changing the namespace prefix also affects the nodeName,
   * tagName and name attributes.<br>
   * DOM-Level 2
   *
   * @public
   * @param   $prefix <code>string</code>
   * @returns void
   */
  function setPrefix($prefix) {
    if (empty($this->namespaceURI)) {
      return NAMESPACE_ERR;
    }

    if (($this->namespaceURI != "http://www.w3.org/XML/1998/namespace") && 
        ($prefix == "xml")) {
      return NAMESPACE_ERR;
    }

    if ($this->nodeType == ATTRIBUTE_NODE) {
      if (($this->namespaceURI != "http://www.w3.org/2000/xmlns/") && 
          ($prefix == "xmlns")) {
        return NAMESPACE_ERR;
      }

      if ($this->qualifiedName == "xmlns") {
        return NAMESPACE_ERR;
      }
    }

    $this->prefix = $prefix;
    $this->nodeName = $prefix.substr($this->nodeName, 
                                     strpos($this->nodeName, ":"));
  }
  
  /**
   * Indicates whether an Element node has any attributes.<br>
   * DOM-Level 2
   *
   * @public
   * @returns boolean
   */
  function hasAttributes() {
    if (isset($this->attributes) && $this->attributes->getLength() > 0)        {
      return true;
    }
    return false;
  }

  /**
   * Checks to see if a particular DOM feature is available for this 
   * implementation.<br>
   * DOM-Level 2
   *
   * @public
   * @param   $feature <code>string</code>
   * @param   $version <code>string</code>
   * @returns boolean
   */
  function isSupported($feature, $version = "1.0") {
    if (($feature == "XML") || ($feature == "Core")) {
        if (($version == "1.0") || ($version == "2.0")) {
          return true;
        }
    }
    return false;
  }
  
  /**
   * Recursively combines all adjacent Text nodes into a single node; It
   * also removes empty Text nodes from the document tree.<br>
   * DOM-Level 2
   *
   * @public
   * @returns void
   */
  function normalize() {
    if ($this->nodeType == "ELEMENT_NODE" && $this->hasAttribues()) {
      for ($i = 0; $i < $this->attributes->getLength(); $i++) {
        $attr =& $this->attributes->item($i);
        $attr->normalize();
      }
    }
    
    if ($this->hasChildNodes()) {
      $child =& $this->firstChild;
      $prevChild = NULL;
      while (isset($child)) {
        if ($child->hasChildNodes()) {
          $child->normalize();
        }
        
        if (($child->nodeType == TEXT_NODE) &&
            (isset($prevChild)) &&
            ($prevChild->nodeType == TEXT_NODE)) {
          if ($prevChild->getLength() > 0) {
            $prevChild->nextSibling =& $child->nextSibling;
            $prevChild->appendData($child->getData());
            $this->removeChild($child);
            $child =& $prevChild;
          }
        }

	if ($child->nodeType == TEXT_NODE) {
	  if ($child->getLength() == 0) {
	    $prevChild->nextSibling =& $child->nextSibling;
	    $this->removeChild($child);
	      $child =& $prevChild;
	  }
	}

        $prevChild =& $child; 
        $child =& $child->nextSibling;
      }
    }
  }
  
  /**
   * Checks, if the node $child is a permitted child object of the DOM
   * hierarchy. <br>
   *
   * @private
   * @param   $child <a href="Node.html">Node</a>
   * @returns int
   */
  function checkHierarchy(&$child) {
    if ($this->nodeID == $child->nodeID) {
      return HIERARCHY_REQUEST_ERR;
    }

    if ($this->readonly) {
      return NO_MODIFICATION_ALLOWED_ERR;
    }

    if (($this->nodeType == PROCESSING_INSTRUCTION_NODE) ||
        ($this->nodeType == COMMENT_NODE) ||
        ($this->nodeType == DOCUMENT_TYPE_NODE) ||
        ($this->nodeType == TEXT_NODE) ||
        ($this->nodeType == CDATA_SECTION_NODE) ||
        ($this->nodeType == NOTATION_NODE)) {
      return HIERARCHY_REQUEST_ERR;
    }
    
    if ($this->nodeType == ATTRIBUTE_NODE) {
      if (($child->nodeType != ENTITY_REFERENCE_NODE) &&
          ($child->nodeType != TEXT_NODE)) {
        return HIERARCHY_REQUEST_ERR;
      }
    }
    
    if ($this->nodeType == DOCUMENT_NODE) {
      if (($child->nodeType != ELEMENT_NODE) &&
          ($child->nodeType != PROCESSING_INSTRUCTION_NODE) &&
          ($child->nodeType != COMMENT_NODE) &&
          ($child->nodeType != DOCUMENT_TYPE_NODE)) {
        return HIERARCHY_REQUEST_ERR;
      }
    }
    
    if (($this->nodeType == DOCUMENT_FRAGMENT_NODE) || 
        ($this->nodeType == ENTITY_REFERENCE_NODE) || 
        ($this->nodeType == ENTITY_NODE) || 
        ($this->nodeType == ELEMENT_NODE)) {
      if (($child->nodeType != ELEMENT_NODE) &&
          ($child->nodeType != PROCESSING_INSTRUCTION_NODE) &&
          ($child->nodeType != COMMENT_NODE) &&
          ($child->nodeType != CDATA_SECTION_NODE) &&
          ($child->nodeType != ENTITY_REFERENCE_NODE) &&
          ($child->nodeType != TEXT_NODE)) {
        return HIERARCHY_REQUEST_ERR;
      }
    }
    
    return 0;
  }

  /**
   * Checks, if the node $child was created from the same document
   * than this node.<br>
   *
   * @private
   * @param   $child <a href="Node.html">Node</a>
   * @returns int
   */
  function checkDocument(&$child) {
    if (($child->ownerDocument == null)) {
      return 0;
    }
    
    if (($child->ownerDocument->nodeID != $this->ownerDocument->nodeID) &&
        (($this->nodeType != DOCUMENT_NODE) && 
         ($this->nodeType != DOCUMENT_TYPE_NODE))) {
      return WRONG_DOCUMENT_ERR;
    }

    return 0;
  }
}
?>
