<?php
// phpXMLDOM - an XML DOM Implementation
//
// This Software and all associated files are released unter the 
// GNU Public License (GPL), see LICENSE for details.
//
// $Id: NodeList.php,v 1.5 2002/02/13 21:21:47 thomi Exp $

/**
 * Class representing the DOM NodeList interface
 *
 * @package  phpXD
 * @author   Thomas Dohmke <thomas@dohmke.de>
 * @version  $Revision: 1.5 $
 */
class NodeList {
  /**
   * Contains all items of tbe NodeList.
   *
   * @private
   * @type    array
   */
  var $nodes; 
  
  /**
   * The number of nodes in this list.<br>
   * DOM-Level 1
   *
   * @private
   * @type    int
   */
  var $length = 0;
  
  /**
   * Returns the number of nodes in the list.<br>
   * DOM-Level 1
   *
   * @public
   * @returns int
   */
  function getLength() {
    return $this->length;
  }
  
  /**
   * Returns the indexth item in this list.<br> 
   * DOM-Level 1
   *
   * @public
   * @param   $index <code>int</code>
   * @returns Node
   */
  function &item($index) {
    return $this->nodes[$index];
  }
  
  /**
   * Insert a Node.
   *
   * @private
   * @param   $node  <a href="Node.html">Node</a>
   * @param   $index <code>int</code>
   * @returns void
   */
  function insertNode(&$node, $index = -1) {
    if (($index == -1) || !(is_array($this->nodes)) || 
        ($index >= count($this->nodes))) {
      $this->nodes[] = &$node;
    }
    else {
      for ($key = 0; $key < count($this->nodes); $key++) {
        if ($key == $index) {
          $save = &$this->nodes[$key];
          $this->nodes[$key] = &$node;
        }
        if ($key > $index) {
          $savenext = &$this->nodes[$key];
          $this->nodes[$key] = &$save;
          $save = &$savenext;
        }
      }
      $this->nodes[] = &$save;
    }
    $this->length++;
  }

  /**
   * Removes a Node.
   *
   * @private
   * @param   $index <code>int</code>
   * @returns void
   */
  function removeNode($index) {
    for ($key = 0; $key < count($this->nodes); $key++) {
      if ($key > $index) {
        $this->nodes[$key-1] = &$this->nodes[$key];
      }
    }
    array_pop($this->nodes);
    $this->length--;
  }

  /**
   * Replaces the indexth node with $node.
   *
   * @private
   * @param   $node  <a href="Node.html">Node</a>
   * @param   $index <code>int</code>
   * @returns void
   */
  function replaceNode(&$node, $index) {
    $this->nodes[$index] = &$node;
  }
}
?>
