<?php
// phpXMLDOM - an XML DOM Implementation
//
// This Software and all associated files are released unter the 
// GNU Public License (GPL), see LICENSE for details.
//
// $Id: DTDParser.php,v 1.6 2002/02/13 21:21:48 thomi Exp $

/**
 * Parse a dtd. EXPERIMENTAL!
 *
 * @package phpXD
 * @author  Thomas Dohmke <thomas@dohmke.de>
 * @version $Revision: 1.6 $
 */
class DTDParser {
  // public -- NamedNodeMaps
  var $elements;
  var $entities;
  var $notations;

  // private
  var $parseDTD = false;
  var $parseInternalDTD = false;
  var $DTDTokens;
  var $parameterEntities;
  var $parameterEntitiesDefined;

  // private -- only valid while parsing
  var $document;
  var $currentElement;
  var $attLists;
  var $currentAttList;

  function DTDParser() {
    $this->document = new Document();
  }

  function parse($str, $dtdonly = true) {
    if (!empty($str)) {
      // Make dummy xml string to use expat as parser/scanner.
      if ($dtdonly) {
	$str = "<!DOCTYPE null [".$str."]><null />";
      }

      // search and replace parameter entities
      while (preg_match('=(.*)\%([a-z,A-Z,0-9,\.]*);(.*)$=sU', $str, $ent)) {
	// Just a hack, cause it seems that expat doesn't support IGNORE or
	// INCLUDE.
	$str = preg_replace('=<!--.*-->=sU', '', $str);
	$str = preg_replace('=<!\[IGNORE\[.*\]\]>=sU', '', $str);
	$str = preg_replace('=<!\[INCLUDE\[(.*)\]\]>=sU', '\\1', $str);

	// <!ENTITY % name PUBLIC publicId systemid>
 	while (preg_match('=(.*)<!ENTITY[ ,\n,\r,\t]*\%[ ,\n,\r,\t]*'.
			  '([a-z,A-Z,0-9,\.]*)[ ,\n,\r,\t]*PUBLIC'.
			  '[ ,\n,\r,\t]*"([a-z,A-Z,0-9,\-,/, ]*)"'.
			  '[ ,\n,\r,\t]*"([a-z,A-Z,0-9,\.,\-,_,/]*)"'.
			  '[ ,\n,\r,\t]*>(.*)$=sU', $str, $ent)) {
	  if (!isset($this->parameterEntitiesDefined[$ent[2]]) ||
	      ($this->parameterEntitiesDefined[$ent[2]] < 2)) {
	    if (isset($this->parameterEntitiesDefined[$ent[2]]) &&
		($this->parameterEntitiesDefined[$ent[2]] == 1)) {
	      // redefined parameter entity => replace current string
	      foreach ($this->parameterEntities as $entref => $body) {
		$ent[1] = str_replace("%".$entref.";", $body, $ent[1]);
	      }
	    }
	    $filename = $this->publicId2Filename($ent[3], $ent[4]);
	    if ($this->file_exists($filename)) {
	      $this->parameterEntities[$ent[2]] = 
		 preg_replace('=<!--.*-->=sU', '', implode("", 
							   file($filename)));
	      $this->parameterEntitiesDefined[$ent[2]] = 1;
	    }
	  }
 	  $str = $ent[1].$ent[5];
 	}
	// <!ENTITY % name SYSTEM systemid>
 	while (preg_match('=(.*)<!ENTITY[ ,\n,\r,\t]*\%[ ,\n,\r,\t]*'.
			  '([a-z,A-Z,0-9,\.]*)[ ,\n,\r,\t]*SYSTEM'.
			  '[ ,\n,\r,\t]*"([a-z,A-Z,0-9,\.,\-,_,/]*)"'.
			  '[ ,\n,\r,\t]*>(.*)$=sU', $str, $ent)) {
 	  if ($this->file_exists($ent[4])) {
	    if (!isset($this->parameterEntitiesDefined[$ent[2]]) ||
		($this->parameterEntitiesDefined[$ent[2]] < 2)) {
	      if (isset($this->parameterEntitiesDefined[$ent[2]]) &&
		  ($this->parameterEntitiesDefined[$ent[2]] == 1)) {
		// redefined parameter entity => replace current string
		foreach ($this->parameterEntities as $entref => $body) {
		  $ent[1] = str_replace("%".$entref.";", $body, $ent[1]);
		}
	      }
	      $this->parameterEntities[$ent[2]] = 
		 preg_replace('=<!--.*-->=sU', '', 
			      implode("", file($ent[3])));
	      $this->parameterEntitiesDefined[$ent[2]] = 1;
	    }
	  }
 	  $str = $ent[1].$ent[4];
 	}
	// <!ENTITY % name body>
	while (preg_match('=(.*)<!ENTITY[ ,\n,\r,\t]*\%[ ,\n,\r,\t]*'.
			  '([a-z,A-Z,0-9,\.]*)[ ,\n,\r,\t]*"(.*)"'.
			  '[ ,\n,\r,\t]*>(.*)$=sU', $str, $ent)) {
	  if (!isset($this->parameterEntitiesDefined[$ent[2]]) ||
	      ($this->parameterEntitiesDefined[$ent[2]] < 2)) {
	    if (isset($this->parameterEntitiesDefined[$ent[2]]) &&
		($this->parameterEntitiesDefined[$ent[2]] == 1)) {
	      // redefined parameter entity => replace current string
	      foreach ($this->parameterEntities as $entref => $body) {
		$ent[1] = str_replace("%".$entref.";", $body, $ent[1]);
	      }
	    }
	    $this->parameterEntities[$ent[2]] = $ent[3];
	    $this->parameterEntitiesDefined[$ent[2]] = 1;
	  }
	  $str = $ent[1].$ent[4];
	}
	foreach ($this->parameterEntities as $entref => $body) {
	  $str = str_replace("%".$entref.";", $body, $str);
 	}

	// Just a hack, cause it seems that expat doesn't support IGNORE or
	// INCLUDE.
	$str = preg_replace('=<!--.*-->=sU', '', $str);
	$str = preg_replace('=<!\[IGNORE\[.*\]\]>=sU', '', $str);
	$str = preg_replace('=<!\[INCLUDE\[(.*)\]\]>=sU', '\\1', $str);
      }

      // Use expat as parser/scanner for finding syntax errors in the dtd.
      // So an array of tokens is provided, which is used as input for
      // the parseToken method.
      $parser = xml_parser_create();
      
      xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
      xml_set_object($parser, $this);
      // Only interested in the dtd.
      xml_set_default_handler($parser, "handleDefault");
      
      if (!xml_parse($parser, $str, true)) {
	$this->displayError("XML error in line %d column %d: %s", 
			    xml_get_current_line_number($parser), 
			    xml_get_current_column_number($parser), 
			    xml_error_string(xml_get_error_code($parser)));
      }
      
      xml_parser_free($parser);
    }
    else {
      $this->displayError("The string is empty.");
    }
  }

  function parseFile($filename, $dtdonly = true) {
    if ($this->file_exists($filename)) {
      $path = explode("/", $filename);
      if (count($path) > 0) {
	unset($path[count($path) - 1]);
	$this->currentDir = implode("/", $path);
      }
      $content = implode("", file($filename));
      return $this->parse($content, $dtdonly);
    }
    else {
      $this->displayError("File %s could not be found or read.", $filename);
    }
  }

  function parseTokens($tokens) {
    // assert: tokens are all valid
    $this->parseDoctype($tokens);

    if (isset($this->elements)) {
      for ($i = 0; $i < $this->elements->getLength(); ++$i) {
	$element =& $this->elements->item($i);
	if (isset($this->attLists[$element->nodeName])) {
	  $element->attributes =& $this->attLists[$element->nodeName];
	}
      }
    }
    else {
      $this->elements = new NamedNodeMap();
    }
  }

  function parseDoctype($tokens) {
    $token = $tokens[0];
    if ($token == "<!DOCTYPE") {
      $this->parseDocumentElement($tokens, 1);
    }
  }

  function parseDocumentElement($tokens, $offset) {
    $token = $tokens[$offset];
    $this->documentElement = $token;
    $token = $tokens[$offset + 1];
    if ($token == "[") {
      $this->parseInternalDTD($tokens, $offset + 2);
      return;
    }
    if ($token == "SYSTEM") {
      $this->parseExternalSystemDTD($tokens, $offset + 2);
      return;
    }
    if ($token == "PUBLIC") {
      $this->parseExternalPublicDTD($tokens, $offset + 2);
      return;
    }
  }

  function parseInternalDTD($tokens, $offset) {
    $token = $tokens[$offset];
    switch ($token) {
    case "<!ELEMENT": {
      $this->parseElement($tokens, $offset + 1);
      // copy element
      if (!isset($this->elements)) {
	$this->elements = new NamedNodeMap();
      }
      $currentElement = $this->currentElement;
      $currentElement->childNodes =& $this->currentElement->childNodes;
      $this->currentElement = null;
      $this->elements->setNamedItem($currentElement);
      $offset = $this->skipToGt($tokens, $offset + 1);
      break;
    }
    case "<!ATTLIST": {
      $this->parseAttList($tokens, $offset + 1);
      $offset = $this->skipToGt($tokens, $offset + 1);
      $this->attLists[$this->currentAttList->element] =&
	 $this->currentAttList->attributes;
      break;
    }
    case "<!ENTITY": {
      $this->parseEntity($tokens, $offset + 1);
      $offset = $this->skipToGt($tokens, $offset + 1);
      break;
    }
    case "<!NOTATION": {
      $this->parseNotation($tokens, $offset + 1);
      $offset = $this->skipToGt($tokens, $offset + 1);
      break;
    }
    }
    if (($tokens[$offset] != "]>") && ($tokens[$offset] != "]")) {
      $this->parseInternalDTD($tokens, $offset);
    }
  }

  function skipToGt($tokens, $offset) {
    while ($tokens[$offset] != ">") {
      $offset++;
    }
    return $offset + 1;;
  }

  function parseExternalSystemDTD($tokens, $offset) {
    $token = $this->removeQuotes($tokens[$offset]);
    $DTDParser = new DTDParser();
    $DTDParser->parameterEntities =& $this->parameterEntities;
    if ((isset($this->parameterEntitiesDefined)) &&
	(is_array($this->parameterEntitiesDefined))) {
      foreach ($this->parameterEntitiesDefined as $key => $value) {
	// internal parameter entities overwrite external
	$DTDParser->parameterEntitiesDefined[$key] = 2;
      }
    }
    $DTDParser->attLists =& $this->attLists;
    $DTDParser->entities =& $this->entities;
    $DTDParser->elements =& $this->elements;
    $DTDParser->notations =& $this->notations;
    $DTDParser->parseFile($token, true);
    
    // merge definitions from $DTDParser with this
    $token = $tokens[$offset + 1];
    if ($token == "[") {
      $this->parseInternalDTD($tokens, $offset + 2);
    }
  }

  function parseExternalPublicDTD($tokens, $offset) {
    $publicId = $this->removeQuotes($tokens[$offset]);
    $systemId = $this->removeQuotes($tokens[$offset + 1]);
    $filename = $this->publicId2Filename($publicId, $systemId);
    $DTDParser = new DTDParser();
    $DTDParser->parameterEntities =& $this->parameterEntities;
    if ((isset($this->parameterEntitiesDefined)) &&
	(is_array($this->parameterEntitiesDefined))) {
      foreach ($this->parameterEntitiesDefined as $key => $value) {
	// internal parameter entities overwrite external
	$DTDParser->parameterEntitiesDefined[$key] = 2;
      }
    }
    $DTDParser->attLists =& $this->attLists;
    $DTDParser->entities =& $this->entities;
    $DTDParser->elements =& $this->elements;
    $DTDParser->notations =& $this->notations;
    $DTDParser->parseFile($filename, true);

    // merge definitions from $DTDParser with this
    $token = $tokens[$offset + 2];
    if ($token == "[") {
      $this->parseInternalDTD($tokens, $offset + 2);
    }
  }

  function parseElement($tokens, $offset) {
    $name = $tokens[$offset];
    $this->currentElement = new DTDElement($name);
    if ($tokens[$offset + 1] == "ANY") {
      $this->currentElement->children = 
	 new DTDElementChild("", "", true);
      return;
    }
    if ($tokens[$offset + 1] == "EMPTY") {
      $this->currentElement->children = null;
      return;
    }
    $this->parseElementChilds($tokens, $offset + 1);
  }

  function parseElementChilds($tokens, $offset) {
    $token = $tokens[$offset];
    if ($token == "(") {
      $nestlevel = 1; $cursor = 1;
      while (($nestlevel > 0) && 
	     (($tokens[$offset + $cursor] != ",") || ($nestlevel > 1))) {
	if ($tokens[$offset + $cursor] == "(") {
	  $nestlevel++;
	}
	if (substr($tokens[$offset + $cursor], 0, 1) == ")") {
	  $nestlevel--;
	}
	$cursor++;
      }
      if ($nestlevel == 1) {
	if ($this->currentElement->childNodes == null) {
	  $this->currentElement->childNodes = new DTDElementSequence();
	} 
	else {
	  $newChild = new DTDElementSequence();
	  $newChild->parent =& $this->currentElement->childNodes;
	  $newChild->parent->insertNode($newChild);
	  unset($this->currentElement->childNodes);
	  $this->currentElement->childNodes =& $newChild;
	}
      } 
      else {
	if ($this->currentElement->childNodes == null) {
	  $this->currentElement->childNodes = new DTDElementChoice();
	}
	else {
	  $newChild = new DTDElementChoice();
	  $newChild->parent =& $this->currentElement->childNodes;
	  $newChild->parent->insertNode($newChild);
	  unset($this->currentElement->childNodes);
	  $this->currentElement->childNodes =& $newChild;
	}
      }
      $this->parseElementChilds($tokens, $offset + 1);
      return;
    }
    if ($token[0] == ")") {
      if (($token[strlen($token) - 1] == "?") || 
	  ($token[strlen($token) - 1] == "+") || 
	  ($token[strlen($token) - 1] == "*")) {
	$this->currentElement->childNodes->setNumber($token[strlen($token) - 
							    1]);
      }
      if ($this->currentElement->childNodes->parent != null) {
	$parent =& $this->currentElement->childNodes->parent;
  	$this->currentElement->childNodes =& $parent;
	$this->parseElementChilds($tokens, $offset + 1);
	return;
      }
      else {
	// assert: the dtd is correct, so it is the last return.
	return;
      }
    }
    if (($token == ",") || ($token == "|") ||
	($token == "?") || ($token == "+") || ($token == "*")) {
      $this->parseElementChilds($tokens, $offset + 1);
      return;
    }
    if ($token == ">") {
      return;
    }
    $this->parseElementChild($tokens, $offset);
    $this->parseElementChilds($tokens, $offset + 1);
  }

  function parseElementChild($tokens, $offset) {
    $token = $tokens[$offset];
    if ($token == "#PCDATA") {
      if ($this->currentElement->childNodes == null) {
	$this->currentElement->childNodes = 
	   new DTDElementChild("", "", false, true);
	return;
      }
      else {
	$this->currentElement->childNodes->
	   insertNode(new DTDElementChild("", "", false, true));
	return;
      }
    }
    if (($token[strlen($token) - 1] == "?") || 
	($token[strlen($token) - 1] == "+") || 
	($token[strlen($token) - 1] == "*")) {
      $number = $token[strlen($token) - 1];
      $token = substr($token, 0, strlen($token) - 1);
    }
    else {
      $number = "";
    }
    $child = new DTDElementChild($token, $number);
    if ($this->currentElement->childNodes == null) {
      $this->currentElement->childNodes =& $child;
      return;
    }
    else {
      $this->currentElement->childNodes->insertNode($child);
      return;
    }
  }

  function parseAttList($tokens, $offset) {
    $name = $tokens[$offset];
    $this->currentAttList = new DTDAttList($name);
    $this->parseAttributes($tokens, $offset + 1);
  }

  function parseAttributes($tokens, $offset) {
    $count = 0;
    while ($tokens[$offset] != ">") {
      $name = $tokens[$offset];

      // TODO: check xml:space and xml:lang attributes for
      // correct implementation
      // ...

      $type = $tokens[$offset + 1];
      if ($type == "(") {
	$attribute[$count] = new DTDAttribute($name, "ENUMERATION");
	// Enumeration
	$offset += 2;
	while ($tokens[$offset] != ")") {
	  if ($tokens[$offset] != "|") {
	    $attribute[$count]->value[] = $tokens[$offset];
	  }
	  $offset++;
	}
      }
      else {
	$attribute[$count] = new DTDAttribute($name, $type);
	$offset++;
      }
      $default = $tokens[$offset + 1];
      $attribute[$count]->default = $default;
      if (($default != "#IMPLIED") &&
	  ($default != "#REQUIRED")) {
	if ($default == "#FIXED") {
	  $attribute[$count]->defaultValue = 
	     $this->removeQuotes($tokens[$offset + 2]);
	  $offset++;
	}
	else {
	  $attribute[$count]->defaultValue = 
	     $this->removeQuotes($tokens[$offset + 1]);
	}
      }
      $offset += 2;
      if (!isset($this->currentAttList->attributes)) {
	$this->currentAttList->attributes = new NamedNodeMap();
      }
      $this->currentAttList->attributes->setNamedItem($attribute[$count]);
      $attribute = null;
      $count++;
    }
  }

  function parseEntity($tokens, $offset) {
    $entity = new Entity();
    $entity->ownerDocument =& $this->document;
    $entity->nodeName = $tokens[$offset];
    $value = $tokens[$offset + 1];
    $entityParser = new DOMParser();
    if (($value == "SYSTEM") || ($value == "PUBLIC")) {
      if ($value == "SYSTEM") {
	$entity->systemId = $this->removeQuotes($tokens[$offset + 2]);
	$offset = $offset + 3;

	$entityDocument =& 
	   $entityParser->parseExternalEntity($entity->systemId);
      }
      else {
	$entity->publicId = $this->removeQuotes($tokens[$offset + 2]);
	$entity->systemId = $this->removeQuotes($tokens[$offset + 3]);
	$offset = $offset + 4;

	$entityDocument =& 
	   $entityParser->parseExternalEntity(
		   $this->publicId2Filename($entity->publicId, 
					    $entity->systemId));
      }
      if ($tokens[$offset] == "NDATA") {
	$entity->notationName = $tokens[$offset + 1];
      }
    }
    else {
      $entityDocument =& 
	 $entityParser->parseEntity($this->removeQuotes($value));
    }
    // parsed entity?
    if ($entity->notationName == null) {
      $entityNodes =& $entityDocument->documentElement->childNodes;
      for ($i = 0; $i < $entityNodes->length; ++$i) {
	$node =& $entityNodes->item($i);
	$entity->appendChild($this->document->importNode($node,
							 true));
      }
    }
    if (!isset($this->entities)) {
      $this->entities = new NamedNodeMap();
    }
    $this->entities->setNamedItem($entity);
  }

  function parseNotation($tokens, $offset) {
    $notation = new Notation();
    $notation->nodeName = $tokens[$offset];
    $notation->ownerDocument =& $this->document;
    $value = $tokens[$offset + 1];
    if ($value == "SYSTEM") {
      $notation->systemId = $this->removeQuotes($tokens[$offset + 2]);
    }
    if ($value == "PUBLIC") {
      $notation->publicId = $this->removeQuotes($tokens[$offset + 2]);
      if ($tokens[$offset + 3] != ">") {
	$notation->systemId = $this->removeQuotes($tokens[$offset + 3]);
      }
    }
    if (!isset($this->notations)) {
      $this->notations = new NamedNodeMap();
    }
    $this->notations->setNamedItem($notation);
  }

  function publicId2Filename($publicId, $systemId) {
    return $systemId;
  }

  function removeQuotes($str) {
    if ((($str[0] == "\"") && ($str[strlen($str) - 1] == "\"")) ||
	(($str[0] == "\'") && ($str[strlen($str) - 1] == "\'"))) {
      return substr($str, 1, strlen($str) - 2);
    }
    return $str;
  }


  function handleDefault($parser, $data) {
    $data = trim($data);

    if ($data == "<!DOCTYPE") {
      $this->DTDTokens[] = $data;
      $this->parseDTD = true;
      return true;
    }

    if ($this->parseDTD) {
      if ($data != "") {
	$this->DTDTokens[] = $data;
      }
      if ($data == "[") {
	$this->parseInternalDTD = true;
      }
      if ($data == "]") {
	$this->parseInternalDTD = false;
      }
      if (($data == ">") && (!$this->parseInternalDTD)) {
	$this->parseDTD = false;
	$this->parseTokens($this->DTDTokens);
      }
      return true;
    }
  }

  function file_exists($filename) {
    $file = @fopen($filename, "r");
    if (!$file) {
      return false;
    }
    return true;
  }

  /**
   * Displays errors, which occur while parsing.
   *
   * @private
   * @returns void
   */
  function displayError($message) {
    if (func_num_args() > 1) {
      $arguments = func_get_args();
      
      $command = "\$message = sprintf(\$message, ";
      
      for ($i = 1; $i < sizeof($arguments); $i++ ) {
        $command .= "\$arguments[".$i."], ";
      }

      $command = eregi_replace(", $", ");", $command);
      
      eval($command);
    }

    echo "<strong>DTDParser error:</strong> ".$message;
    exit;
  }
}
    
